#include <avr/io.h>
#include <util/delay.h>

#define N	14

#define LEFT_SIDE	1
#define RIGHT_SIDE	2

#define FORWARD	0
#define LEFT	1
#define BACK	3
#define STOP	5

int ADC_get_value_sensor(void)
{
	/* declansare conversie */
	ADCSRA |= (1<<ADSC);
	/* asteptare terminare conversie */
	while( (ADCSRA & (1<<ADIF)) == 0);
	
	return ADC;
}


int distance(int v0)
{

	int d[] = {15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80};
	int v[] = {358,492,540,496,444,393,344,305,276,246,229,215,195,186,170,165};
	
	int i;
	for(i=2; i<N-1; i++)
		if(v0 < v[i] && v0 > v[i+1])
			return (v0-v[i+1])*(d[i]-d[i+1])/(v[i]-v[i+1]) + d[i+1];
	return 150;
}


void move_side(int side, int direction) {

	if(side == LEFT_SIDE) {
		switch(direction) {
			case FORWARD:
				PORTC &=~(1<<PC1);
				PORTC |= (1<<PC0);
				break;
			case BACK:
				PORTC &=~(1<<PC0);
				PORTC |= (1<<PC1);
				break;
			case STOP:
				PORTC &=~(1<<PC0);
				PORTC &=~(1<<PC1);
				break;
		}
	}
	else if(side == RIGHT_SIDE) {
		switch(direction) {
			case FORWARD:
				PORTC &=~(1<<PC3);
				PORTC |= (1<<PC2);
				break;
			case BACK:
				PORTC &=~(1<<PC2);
				PORTC |= (1<<PC3);
				break;
			case STOP:
				PORTC &=~(1<<PC2);
				PORTC &=~(1<<PC3);
				break;
		}
	}
}

void move(int directie, int time) {

	switch(directie) {
		case FORWARD:
			move_side(LEFT_SIDE,FORWARD);
			move_side(RIGHT_SIDE,FORWARD);
			break;
		case LEFT:
			move_side(LEFT_SIDE,BACK);
			move_side(RIGHT_SIDE,FORWARD);			
			break;
		case STOP:
			move_side(LEFT_SIDE,STOP);
			move_side(RIGHT_SIDE,STOP);
			break;
			
	}
	
	_delay_ms(time);
}

int main() {
	
	DDRA = 0x00; // port A - pini intrare
	DDRC = 0xFF; // port C - pini iesire
	PORTC = 0x00;
	
	// init ADC - take data
	ADMUX = 0x00;
	/* select reference Vcc */
	ADMUX &= ~(1<<REFS1);
	ADMUX |= (1<<REFS0);
	/* select LSB order */
	ADMUX &= ~(1<<ADLAR);
	
	/* Enable conversie + conversie in trigger mode */
	ADCSRA |= (1<<ADEN);
	
	ADCSRA |= (1<<ADPS0) | (1<<ADPS1) | (1<<ADPS2);
	/* prima conversie este pentru init */
	ADCSRA |= (1<<ADSC);
	
	// init PWM - use data
	/* set pin out pentru OC0 */
	DDRB |= (1<<PB3);
	
	/* mod Fast PWM cu 1024 prescaler */
	TCCR0 |= (1<<WGM01)|(1<<WGM00);
	TCCR0 |= (1<<CS02)|(1<<CS00);
	TCCR0 &= ~(1<<CS01);
	/* setare non inverting mode */
	TCCR0 |= (1<<COM01);
	TCCR0 &= ~(1<<COM00);
	
	/* set pin out pentru OC2 */
	DDRD |= (1<<PD7);
	/* aceleasi setari ca pentru primul PWM */
	TCCR2 |= (1<<WGM21)|(1<<WGM20);
	TCCR2 |= (1<<CS20)|(1<<CS21)|(1<<CS22);

	TCCR2 |= (1<<COM21);
	TCCR2 &= ~(1<<COM20);
	
	
	OCR0 = 170;
	OCR2 = 170;
	
	while(1) {
		
		
		if(distance(ADC_get_value_sensor()) < 30) {
			move(STOP, 1000);
			move(LEFT, 5000);
			move(STOP, 3000);
		}
		else {
			move(FORWARD,0);
		}
	
	}
	
	return 0;
}
